@@ -25,6 +25,7 @@ SV *Py2Pl(PyObject * obj) {
         PyObject *this_type = PyObject_Type(obj);
         PyObject *t_string = PyObject_Str(this_type);
         char *type_str = PyString_AsString(t_string);
+        Py_DECREF(t_string);
         Printf(("type is %s\n", type_str));
 #ifdef I_PY_DEBUG
 	printf("Py2Pl object:\n\t");
@@ -52,6 +53,7 @@ SV *Py2Pl(PyObject * obj) {
 	if ((obj->ob_type->tp_flags & Py_TPFLAGS_HAVE_CLASS))
 		printf("has class\n");
 #endif
+        Py_DECREF(this_type);
 	/* elw: this needs to be early */
 	/* None (like undef) */
 	if (!obj || obj == Py_None) {
@@ -71,7 +73,7 @@ SV *Py2Pl(PyObject * obj) {
 	else if (PerlSubObject_Check(obj)) {
 		Printf(("Py2Pl: Sub_object\n"));
 		SV *ref = ((PerlSub_object *) obj)->ref;
-		if (! ref) { // probably an inherited method
+		if (! ref) { /* probably an inherited method */
 			if (! ((PerlSub_object *) obj)->obj)
 				croak("Error: could not find a code reference or object method for PerlSub");
 			SV *sub_obj = (SV*)SvRV(((PerlSub_object *) obj)->obj);
@@ -106,13 +108,13 @@ SV *Py2Pl(PyObject * obj) {
 
 		/* set up magic */
 		priv.key = INLINE_MAGIC_KEY;
-		sv_magic(inst, inst, '~', (char *) &priv, sizeof(priv));
-		mg = mg_find(inst, '~');
+		sv_magic(inst, inst, PERL_MAGIC_ext, (char *) &priv, sizeof(priv));
+		mg = mg_find(inst, PERL_MAGIC_ext);
 		mg->mg_virtual = (MGVTBL *) malloc(sizeof(MGVTBL));
 		mg->mg_virtual->svt_free = free_inline_py_obj;
 
 		sv_setiv(inst, (IV) obj);
-		/*SvREADONLY_on(inst); *//* to uncomment this means I can't
+		/*SvREADONLY_on(inst); */ /* to uncomment this means I can't
 			re-bless it */
 		Py_INCREF(obj);
 		Printf(("Py2Pl: Instance. Obj: %p, inst_ptr: %p\n", obj, inst_ptr));
@@ -232,7 +234,7 @@ SV *Py2Pl(PyObject * obj) {
 		mg->mg_virtual->svt_free = free_inline_py_obj;
 
 		sv_setiv(inst, (IV) obj);
-		/*SvREADONLY_on(inst); *//* to uncomment this means I can't
+		/*SvREADONLY_on(inst); */ /* to uncomment this means I can't
 			re-bless it */
 		Py_INCREF(obj);
 		Printf(("Py2Pl: Instance. Obj: %p, inst_ptr: %p\n", obj, inst_ptr));
